#!/bin/bash
# Builds a arm-elf cross compiler for use on linux
# host systems. This target has no file system or kernel.
#
# -- define version
VER_BINUTILS=2.18
VER_GCC=4.5.1
VER_GMP=5.0.2
VER_MPFR=3.1.0
# filename or nothing
PATCH_MPFR= #allpatches
VER_MPC=0.9
VER_NEWLIB=1.19.0
# -- define dirs
TARGET="arm-elf"
TOOLDIR="${TARGET}-${VER_GCC//.}"
TARNAME="${TOOLDIR}_${VER_BINUTILS//.}"
TOPDIR=$(pwd)
SRC="${TOPDIR}/packages" # package directory
BUILDDIR="${TOPDIR}/build-dir"
TEMPDIR="${BUILDDIR}/temp"
PREFIX="${BUILDDIR}/${TOOLDIR}" # final toolchain directory
BINUTILSINSTALL="${BUILDDIR}/binutilsinstall" # flag file for installed binutils in toolchain dir
PARALLEL="-j 2"
# -- define packages and configuration
BINUTILS=binutils-${VER_BINUTILS}
GCC=gcc-${VER_GCC}
GMP=gmp-${VER_GMP}
MPFR=mpfr-${VER_MPFR}
MPC=mpc-${VER_MPC}
NEWLIB=newlib-${VER_NEWLIB}
BASE_CONF="--target=${TARGET} --prefix=${PREFIX} \
--enable-interwork --enable-multilib --disable-nls \
--with-float=soft"
BINUTILS_CONF="${BASE_CONF} --disable-werror"
GCC_CONF="${BASE_CONF} --enable-languages=c --with-newlib --with-system-zlib \
--disable-shared --disable-libc --disable-libm --disable-intl --disable-threads \
--disable-libssp --without-headers --with-gnu-ld --with-gnu-as \
--enable-obsolete" #for gcc 4.7
#
export PATH="$PATH:${PREFIX}/bin"
# --download and prepare packages
if [ ! -d ${SRC} ] ; then
    mkdir ${SRC}
fi
cd ${SRC}
echo "Downloading source packages. This may take a while."
echo
if [ ! -r ${BINUTILS}.tar.bz2 ] ; then
    echo "Download ${BINUTILS}"
    wget -c http://ftp.gnu.org/gnu/binutils/${BINUTILS}.tar.bz2
fi
if [ ! -r ${GCC}.tar.bz2 ] ; then
    echo "Download ${GCC}"
    wget -c ftp://ftp.gnu.org/gnu/gcc/${GCC}/${GCC}.tar.bz2
fi
if [ ! -r ${GMP}.tar.bz2 ] ; then
    echo "Download ${GMP}"
    wget -c ftp://ftp.gmplib.org/pub/${GMP}/${GMP}.tar.bz2
fi
if [ ! -r ${MPFR}.tar.bz2 ] ; then
    echo "Download ${MPFR}"
    wget -c http://www.mpfr.org/${MPFR}/${MPFR}.tar.bz2
fi
if [ ${#PATCH_MPFR} -gt 0 ] && [ ! -r ${PATCH_MPFR} ] ; then
    echo "Download ${PATCH_MPFR}"
    wget -c http://www.mpfr.org/mpfr-current/${PATCH_MPFR}
fi
if [ ! -r ${MPC}.tar.gz ] ; then
    echo "Download ${MPC}"
    wget -c http://www.multiprecision.org/mpc/download/${MPC}.tar.gz
fi
if [ ! -r ${NEWLIB}.tar.gz ] ; then
    echo "Download ${NEWLIB}"
    wget -c ftp://sources.redhat.com/pub/newlib/${NEWLIB}.tar.gz
fi
cd ..
echo
echo "Starting build process for ${TARGET} [$(date)]"
echo
if [ ! -d ${BUILDDIR} ] ; then
    mkdir ${BUILDDIR}
fi
cd ${BUILDDIR}
if [ ! -d ${BINUTILS} ] ; then
    echo "Extract ${BINUTILS}"
    tar -jxf ${SRC}/${BINUTILS}.tar.bz2
#
# makeinfo is required for binutils 2.18
# makeinfo patch for texinfo 4.11, 4.13
    if [ "${VER_BINUTILS}" = "2.18" ] ; then
#
ed ${BUILDDIR}/${BINUTILS}/configure << EOF
6131s/\[4-9\]/(4|5|6|7|8|9|11|13)
wq
EOF
#
    fi
fi
if [ ! -d ${GMP} ] ; then
    echo "Extract ${GMP}"
    tar -jxf ${SRC}/${GMP}.tar.bz2
fi
if [ ! -d ${MPFR} ] ; then
    echo "Extract ${MPFR}"
    tar -jxf ${SRC}/${MPFR}.tar.bz2
    if [ ${#PATCH_MPFR} -gt 0 ] && [ -d ${MPFR} ] && [ -r ${SRC}/${PATCH_MPFR} ] ; then
        echo "Patch ${MPFR} with ${PATCH_MPFR}"
        cd ${MPFR}
        patch -N -Z -p1 < ${SRC}/${PATCH_MPFR}
        cd ..
    fi
fi
if [ ! -d ${MPC} ] ; then
    echo "Extract ${MPC}"
    tar -zxf ${SRC}/${MPC}.tar.gz
fi
if [ ! -d ${NEWLIB} ] ; then
    echo "Extract ${NEWLIB}"
    tar -zxf ${SRC}/${NEWLIB}.tar.gz
fi
if [ ! -d ${GCC} ] ; then
    echo "Extract ${GCC}"
    tar -jxf ${SRC}/${GCC}.tar.bz2

ed ${BUILDDIR}/${GCC}/gcc/config/arm/t-arm-elf << EOF
/mno-thumb-interwork/s/^# //
/normal interwork/s/^# //
wq
EOF
#
    ln -s ${BUILDDIR}/${GMP} ${BUILDDIR}/${GCC}/gmp
    ln -s ${BUILDDIR}/${MPFR} ${BUILDDIR}/${GCC}/mpfr
    ln -s ${BUILDDIR}/${MPC} ${BUILDDIR}/${GCC}/mpc
    ln -s ${BUILDDIR}/${NEWLIB}/newlib ${BUILDDIR}/${GCC}/newlib
fi
if [ ! -d ${TEMPDIR} ] ; then
    mkdir -p ${TEMPDIR}
fi
if [ ! -r ${BINUTILSINSTALL} ] ; then
# -- make binutils
    echo "Building binutils"
    echo
    cd ${TEMPDIR}
    if !(${BUILDDIR}/${BINUTILS}/configure ${BINUTILS_CONF} && make ${PARALLEL} all && make install)
    then
        echo "ERROR in make ${BINUTILS}"
        exit 1
    fi
    rm -rf ${TEMPDIR}/*
    touch ${BINUTILSINSTALL}
fi
# -- make gcc and newlib
echo
echo "Building GCC"
echo
cd ${TEMPDIR}
# -- include and libs path for gmp5, mpfr3 on gcc4.7.0
if !( ${BUILDDIR}/${GCC}/configure ${GCC_CONF} \
--with-gmp-include=${TEMPDIR}/gmp \
--with-gmp-lib=${TEMPDIR}/gmp/.libs \
--with-mpfr-include=${BUILDDIR}/${GCC}/mpfr/src \
--with-mpfr-lib=${TEMPDIR}/mpfr/src/.libs \
&& make ${PARALLEL} all )
then
    echo "ERROR in make ${GCC}"
    exit 1
fi
rm ${BINUTILSINSTALL}
if !( make install )
then
    echo "ERROR in install ${GCC}"
    exit 1
fi
# -- build a toolchain package and clean up
cd ${TOPDIR}
echo
echo "Packing toolchain to ${TARNAME}.tar.gz"
echo
tar -czf ${TARNAME}.tar.gz -C ${BUILDDIR} ${TOOLDIR}
rm -rf ${BUILDDIR}
echo "Finished build process for ${TARGET} [$(date)]"
echo
